package net.wanji.utc.dt.service.controller.impl;

import cn.hutool.core.util.HexUtil;
import net.wanji.common.enums.BaseEnum;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.dt.cache.CrossInfoCache;
import net.wanji.utc.dt.cache.SignalDataCache;
import net.wanji.utc.dt.common.constants.Constants;
import net.wanji.utc.dt.common.enums.ControlModelEnum;
import net.wanji.utc.dt.common.enums.ControlTypeEnum;
import net.wanji.utc.dt.common.enums.PhaseLightColorEnum;
import net.wanji.utc.dt.netty.NettyClient;
import net.wanji.utc.dt.pojo.dtconvert.*;
import net.wanji.utc.dt.pojo.netty.MessageResultPojo;
import net.wanji.utc.dt.service.controller.SignalStatusService;
import net.wanji.utc.dt.util.CommonUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
public class DTSignalStatusServiceImpl implements SignalStatusService {

    @Resource
    private CrossInfoCache crossInfoCache;

    @Override
    public String getBeanMark() {
        return BaseEnum.VendorTypeEnum.DT.getNick();
    }

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) {
        List<String> lightsStatus = getLightsStatusInfoPojos(crossId);
        List<FaultInfoPojo> faultInfoPojos = getFaultInfoPojos(crossId);
        List<RunningStatusPojo> runningStatusPojos = getRunningStatusPojos(crossId);
        return getSignalStatus(crossId, faultInfoPojos, runningStatusPojos);
    }

    private List<SignalStatusLogPO> getSignalStatus(String crossId, List<FaultInfoPojo> faultInfoPojos, List<RunningStatusPojo> runningStatusPojos) {
        List<SignalStatusLogPO> signalStatusLogPOS = new ArrayList<>();
        for (RunningStatusPojo runningStatusPojo : runningStatusPojos) {
            SignalStatusLogPO signalStatusLogPO = new SignalStatusLogPO();
            signalStatusLogPO.setCrossId(crossId);
            signalStatusLogPO.setSignalId(crossId);
            signalStatusLogPO.setStatus(1);
            if (CollectionUtils.isEmpty(faultInfoPojos)) {
                signalStatusLogPO.setFaultType(0);
            } else {
                signalStatusLogPO.setFaultType(1);
            }
            signalStatusLogPO.setControlType(ControlTypeEnum.getWjControlType(runningStatusPojo.getControlMode()));
            signalStatusLogPOS.add(signalStatusLogPO);
        }

        return signalStatusLogPOS;
    }

    private List<RunningStatusPojo> getRunningStatusPojos(String crossId) {
        List<RunningStatusPojo> runningStatusPojos = new ArrayList<>();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_RUNNING_STATUS);
        String runningStatusSign = Constants.COMMAND_RUNNING_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String runnningStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), runnningStatusHex, runningStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int size = body.length() / 14;
            List<String> paramList = new ArrayList<>();
            for (int i = 0; i < size; i++) {
                String param = body.substring(i * 14 + 6, i * 14 + 10);
                if (!StringUtils.equalsIgnoreCase("0000", param)) {
                    paramList.add(param);
                }
            }
            for (int i = 0; i < paramList.size() / 4; i++) {
                RunningStatusPojo runningStatusPojo = new RunningStatusPojo();
                runningStatusPojo.setControlCrossId(HexUtil.hexToInt(paramList.get(i * 4)));
                runningStatusPojo.setControlMode(paramList.get(i * 4 + 1));
                runningStatusPojo.setSchemeId(HexUtil.hexToInt(paramList.get(i * 4 + 2)));
                runningStatusPojo.setPhaseStageId(HexUtil.hexToInt(paramList.get(i * 4 + 3)));
                runningStatusPojos.add(runningStatusPojo);
            }
        }
        return runningStatusPojos;
    }

    private List<String> getLightsStatusInfoPojos(String crossId) {
        List<String> lightsStatusList = new ArrayList<>();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_LIGHT_STATUS);
        String lightStatusSign = Constants.COMMAND_LIGHT_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String lightStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), lightStatusHex, lightStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsStatusSize = body.length() / 14;
            for (int i = 0; i < lightsStatusSize; i++) {
                String param = body.substring(i * 14 + 8, i * 14 + 10);
                if (!StringUtils.equalsIgnoreCase("00", param)) {
                    lightsStatusList.add(param);
                }
            }
        }
        return lightsStatusList;
    }

    private List<FaultInfoPojo> getFaultInfoPojos(String crossId) {
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_FAULT_LIST);
        String faultSign = Constants.COMMAND_FAULT_LIST.concat(Constants.COMMAND_QUERY_REPLY);
        String faultHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), faultHex, faultSign, 300);
        List<FaultInfoPojo> faultInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsStatusSize = body.length() / 14 * 64;
            int paramIndex = 8;
            for (int i = 0; i < lightsStatusSize; i++) {
                String param = body.substring(paramIndex * i, paramIndex * i + 8);


            }

        }
        return null;
    }

    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        List<String> lightsStatus = getLightsStatusInfoPojos(crossId);
        List<PhaseStageStatusPojo> phaseStageStatusPojos = getPhaseStageStatus(crossId);
        List<RunningStatusPojo> runningStatusPojos = getRunningStatusPojos(crossId);
        return getLightsStatusVOs(crossId, lightsStatus, phaseStageStatusPojos, runningStatusPojos);
    }

    private List<PhaseStageStatusPojo> getPhaseStageStatus(String crossId) {
        List<PhaseStageStatusPojo> phaseStageStatusPojos = new ArrayList<>();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_PHASE_STAGE_STATUS);
        String stageStatusSign = Constants.COMMAND_PHASE_STAGE_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String stageStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), stageStatusHex, stageStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int size = body.length() / 14;
            String matchStr = "0603%s%s";
            int index = 0;
            for (int i = 0; i < size; i++) {
                PhaseStageStatusPojo phaseStageStatusPojo = new PhaseStageStatusPojo();
                for (int j = 2; j < 5; j++) {
                    String currentMatchStr = String.format(matchStr, String.format("%02x", j), String.format("%02x", i + 1));
                    index = body.indexOf(currentMatchStr) + 8;
                    if (j == 2) {
                        String stageStatus = body.substring(index, index + 2);
                        phaseStageStatusPojo.setPhaseStageStatus(stageStatus);
                    }
                    if (j == 3) {
                        phaseStageStatusPojo.setRunningTime(HexUtil.hexToInt(body.substring(index, index + 4)));
                    }
                    if (j == 4) {
                        phaseStageStatusPojo.setRemainTime(HexUtil.hexToInt(body.substring(index, index + 4)));
                    }
                }
                phaseStageStatusPojo.setPhaseStageId(i + 1);
                phaseStageStatusPojos.add(phaseStageStatusPojo);
            }
        }
        return phaseStageStatusPojos.stream().filter(phaseStageStatusPojo -> !StringUtils.equals(phaseStageStatusPojo.getPhaseStageStatus(), "10")).collect(Collectors.toList());
    }

    private List<LightsStatusVO> getLightsStatusVOs(String crossId, List<String> lightsStatus, List<PhaseStageStatusPojo> phaseStageStatusPojos, List<RunningStatusPojo> runningStatusPojos) {
        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>(runningStatusPojos.size());
        for (RunningStatusPojo runningStatusPojo : runningStatusPojos) {
            LightsStatusVO lightsStatusVO = new LightsStatusVO();
            String controlModel = String.valueOf(ControlModelEnum.getWjControlMode(runningStatusPojo.getControlMode().substring(2, 4)));
            lightsStatusVO.setCrossId(crossId);
            lightsStatusVO.setRunMode(controlModel);
            lightsStatusVO.setControlMode(controlModel);
            PhaseStageStatusPojo phaseStageStatusPojo = null;
            if (!CollectionUtils.isEmpty(phaseStageStatusPojos)) {
                phaseStageStatusPojo = phaseStageStatusPojos.get(0);
            }
            lightsStatusVO.setCycleCountDown(phaseStageStatusPojo.getRemainTime()); // 相位剩余时长
            Integer currentSchemeId = runningStatusPojo.getSchemeId();
            Integer currentPhaseStageId = runningStatusPojo.getPhaseStageId();
            List<SchemeInfoPojo> schemeInfoPojos = SignalDataCache.schemeInfoCache.get(crossId);
            if (!CollectionUtils.isEmpty(schemeInfoPojos)) {
                for (SchemeInfoPojo schemeInfoPojo : schemeInfoPojos) {
                    if (Objects.equals(schemeInfoPojo.getSchemeId(), currentSchemeId)) {
                        lightsStatusVO.setPlanId(String.valueOf(currentSchemeId));
                        lightsStatusVO.setCycleLen(schemeInfoPojo.getCycle());
                        List<SchemeInfoPojo.SchemeStagePojo> stageInfoList = SchemeInfoPojo.getStageInfoList(schemeInfoPojo);
                        List<PhaseStageInfoPojo> phaseStageInfoPojos = SignalDataCache.phaseStageInfoCache.get(crossId);
                        if (!CollectionUtils.isEmpty(phaseStageInfoPojos)) {
                            for (PhaseStageInfoPojo phaseStageInfoPojo : phaseStageInfoPojos) {
                                if (Objects.equals(phaseStageInfoPojo.getPhaseStageId(), currentPhaseStageId)) {
                                    List<Integer> phaseList = phaseStageInfoPojo.getPhaseList();
                                    String phaseId = StringUtils.join(phaseList, "|");
                                    lightsStatusVO.setPhaseId(phaseId);
                                }
                            }
                        }
                        Map<String, Object> dirLampGroupMap = new HashMap<>();
                        if (!CollectionUtils.isEmpty(lightsStatus)) {
                            for (int i = 0; i < lightsStatus.size(); i++) {
                                Map<Integer, String> turnColorMap = new HashMap<>();
                                String wjLightColor = PhaseLightColorEnum.getWjLightColor(lightsStatus.get(i));
                                Integer turn = CommonUtils.getTurnByLightsId(i);
                                turnColorMap.put(turn, wjLightColor);
                                String dir = String.valueOf(CommonUtils.getDirByLightsId(i));
                                if (dirLampGroupMap.isEmpty() || Objects.isNull(dirLampGroupMap.get(dir))) {
                                    dirLampGroupMap.put(dir, turnColorMap);
                                } else {
                                    Map<Integer, String> currentTurnColorMap = (Map<Integer, String>) dirLampGroupMap.get(dir);
                                    currentTurnColorMap.put(turn, wjLightColor);
                                    dirLampGroupMap.put(dir, currentTurnColorMap);
                                }
                            }
                        }
                        lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
                    }
                }
            }
            lightsStatusVOS.add(lightsStatusVO);
        }
        return lightsStatusVOS;
    }
}
